@@ -3,7 +3,7 @@
 *                            W i n d o w   O b j e c t                          *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,11 +19,16 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXWindow.h,v 1.59 2001/06/15 05:09:47 jeroen Exp $                       *
+* $Id: FXWindow.h,v 1.80 2002/01/18 22:42:55 jeroen Exp $                       *
 ********************************************************************************/
 #ifndef FXWINDOW_H
 #define FXWINDOW_H
 
+#ifndef FXDRAWABLE_H
+#include "FXDrawable.h"
+#endif
+
+
 
 /// Layout hints for child widgets
 enum {
@@ -48,8 +53,8 @@ enum {
   LAYOUT_FIX_HEIGHT  = 0x00000200,                          /// height fixed
   LAYOUT_MIN_WIDTH   = 0,                                   /// Minimum width is the default
   LAYOUT_MIN_HEIGHT  = 0,                                   /// Minimum height is the default
-  LAYOUT_FILL_X      = 0x00000400,                          /// Stretch horizontally
-  LAYOUT_FILL_Y      = 0x00000800,                          /// Stretch vertically
+  LAYOUT_FILL_X      = 0x00000400,                          /// Stretch or shrink horizontally
+  LAYOUT_FILL_Y      = 0x00000800,                          /// Stretch or shrink vertically
   LAYOUT_EXPLICIT    = LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT   /// Explicit placement
   };
 
@@ -126,6 +131,8 @@ protected:
   FXWindow();
   FXWindow(FXApp* a,FXVisual *vis);
   FXWindow(FXApp* a,FXWindow* own,FXuint opts,FXint x,FXint y,FXint w,FXint h);
+  static FXWindow* findDefault(FXWindow* window);
+  static FXWindow* findInitial(FXWindow* window);
   virtual FXbool doesOverrideRedirect() const;
   virtual void layout();
 private:
@@ -137,27 +144,27 @@ protected:
   // Window state flags
   enum {
     FLAG_SHOWN        = 0x00000001,     // Is shown
-    FLAG_FOCUSED      = 0x00000002,     // Has focus
+    FLAG_ENABLED      = 0x00000002,     // Able to receive input
     FLAG_UPDATE       = 0x00000004,     // Is subject to GUI update
-    FLAG_PRESSED      = 0x00000008,     // Button has been pressed
-    FLAG_KEY          = 0x00000010,     // Keyboard key pressed
-    FLAG_ACTIVE       = 0x00000020,     // Window is active
-    FLAG_ENABLED      = 0x00000040,     // Able to receive input
-    FLAG_DIRTY        = 0x00000080,     // Needs layout
-    FLAG_RECALC       = 0x00000100,     // Needs recalculation
-    FLAG_DROPTARGET   = 0x00000200,     // Drop target
-    FLAG_CARET        = 0x00000400,     // Caret is on
-    FLAG_CHANGED      = 0x00000800,     // Window data changed
-    FLAG_TIP          = 0x00001000,     // Show tip
-    FLAG_HELP         = 0x00002000,     // Show help
-    FLAG_LASSO        = 0x00004000,     // Lasso mode
-    FLAG_TRYDRAG      = 0x00008000,     // Tentative drag mode
-    FLAG_DODRAG       = 0x00010000,     // Doing drag mode
-    FLAG_SCROLLINSIDE = 0x00020000,     // Scroll only when inside
-    FLAG_SCROLLING    = 0x00040000,     // Right mouse scrolling
-    FLAG_ROOT         = 0x00080000,     // Root window
-    FLAG_SHELL        = 0x00100000,     // Shell window
-    FLAG_POPUP        = 0x00200000      // Popup window
+    FLAG_DROPTARGET   = 0x00000008,     // Drop target
+    FLAG_FOCUSED      = 0x00000010,     // Has focus
+    FLAG_DIRTY        = 0x00000020,     // Needs layout
+    FLAG_RECALC       = 0x00000040,     // Needs recalculation
+    FLAG_TIP          = 0x00000080,     // Show tip
+    FLAG_HELP         = 0x00000100,     // Show help
+    FLAG_DEFAULT      = 0x00000200,     // Default widget
+    FLAG_INITIAL      = 0x00000400,     // Initial widget
+    FLAG_SHELL        = 0x00000800,     // Shell window
+    FLAG_ACTIVE       = 0x00001000,     // Window is active
+    FLAG_PRESSED      = 0x00002000,     // Button has been pressed
+    FLAG_KEY          = 0x00004000,     // Keyboard key pressed
+    FLAG_CARET        = 0x00008000,     // Caret is on
+    FLAG_CHANGED      = 0x00010000,     // Window data changed
+    FLAG_LASSO        = 0x00020000,     // Lasso mode
+    FLAG_TRYDRAG      = 0x00040000,     // Tentative drag mode
+    FLAG_DODRAG       = 0x00080000,     // Doing drag mode
+    FLAG_SCROLLINSIDE = 0x00100000,     // Scroll only when inside
+    FLAG_SCROLLING    = 0x00200000      // Right mouse scrolling
     };
 
 public:
@@ -229,7 +236,6 @@ public:
     ID_UNKNOWN,
     ID_UPDATE,
     ID_AUTOSCROLL,
-    ID_CARETBLINK,
     ID_HSCROLLED,
     ID_VSCROLLED,
     ID_SETVALUE,
@@ -357,12 +363,6 @@ public:
   /// Set the window height
   void setHeight(FXint h);
 
-  /// Return the width of the widest child window
-  FXint maxChildWidth() const;
-
-  /// Return the height of the tallest child window
-  FXint maxChildHeight() const;
-
   /// Set layout hints for this window
   void setLayoutHints(FXuint lout);
 
@@ -429,15 +429,15 @@ public:
   /// Warp the cursor to the new position
   FXint setCursorPosition(FXint x,FXint y);
 
-  /// Return true if this window is a control capable of receiving the focus
-  virtual FXbool canFocus() const;
-
   /// Return true if this window is able to receive mouse and keyboard events
   FXbool isEnabled() const;
 
   /// Return true if the window is active
   FXbool isActive() const;
 
+  /// Return true if this window is a control capable of receiving the focus
+  virtual FXbool canFocus() const;
+
   /// Return true if this window has the focus
   FXbool hasFocus() const;
 
@@ -447,6 +447,24 @@ public:
   /// Remove the focus from this window
   virtual void killFocus();
 
+  /**
+  * This changes the default window which responds to the Return
+  * key in a dialog. If enable is TRUE, this window becomes the default
+  * window; when enable is FALSE, this window will be no longer the
+  * default window.  Finally, when enable is MAYBE, the default window
+  * will revert to the initial default window.
+  */
+  virtual void setDefault(FXbool enable=TRUE);
+
+  /// Return true if this is the default window
+  FXbool isDefault() const;
+
+  /// Make this window the initial default window
+  void setInitial(FXbool enable=TRUE);
+
+  /// Return true if this is the initial default window
+  FXbool isInitial() const;
+
   /// Enable the window to receive mouse and keyboard events
   virtual void enable();
 
@@ -484,7 +502,7 @@ public:
   void forceRefresh();
 
   /// Change the parent for this window
-  virtual void reparent(FXComposite* newparent);
+  virtual void reparent(FXWindow* newparent);
 
   /// Scroll rectangle x,y,w,h by a shift of dx,dy
   void scroll(FXint x,FXint y,FXint w,FXint h,FXint dx,FXint dy);
@@ -495,6 +513,12 @@ public:
   /// Mark the entire window client area dirty
   void update();
 
+  /// If marked but not yet painted, paint the given area
+  void repaint(FXint x,FXint y,FXint w,FXint h);
+
+  /// If marked but not yet painted, paint the entire window
+  void repaint();
+
   /**
   * Grab the mouse to this window; future mouse events will be
   * reported to this window even while the cursor goes outside of this window
@@ -504,6 +528,18 @@ public:
   /// Release the mouse grab
   void ungrab();
 
+  /// Return true if the window has been grabbed
+  FXbool grabbed() const;
+
+  /// Grab keyboard device
+  void grabKeyboard();
+
+  /// Ungrab keyboard device
+  void ungrabKeyboard();
+
+  /// Return true if active grab is in effect
+  FXbool grabbedKeyboard() const;
+
   /// Show this window
   virtual void show();
 
@@ -513,22 +549,19 @@ public:
   /// Return true if the window is shown
   FXbool shown() const;
 
-  /// Return true if the window has been grabbed
-  FXbool grabbed() const;
-
   /// Return true if the window is composite
   virtual FXbool isComposite() const;
 
   /// Return true if the window is under the cursor
   FXbool underCursor() const;
 
-  /// Return true if this window owns the selection
+  /// Return true if this window owns the primary selection
   FXbool hasSelection() const;
 
-  /// Try to acquire the selection, given a list of drag types
+  /// Try to acquire the primary selection, given a list of drag types
   FXbool acquireSelection(const FXDragType *types,FXuint numtypes);
 
-  /// Release the selection
+  /// Release the primary selection
   FXbool releaseSelection();
 
   /// Return true if this window owns the clipboard
